# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"RegionST": "01_data.ipynb",
         "extract_region": "01_data.ipynb",
         "coords2bbox": "01_data.ipynb",
         "split_region": "01_data.ipynb",
         "merge_tifs": "01_data.ipynb",
         "filter_region": "01_data.ipynb",
         "filter_cloudy": "01_data.ipynb",
         "n_least_cloudy": "01_data.ipynb",
         "download_topography_data": "01_data.ipynb",
         "download_data": "01_data.ipynb",
         "download_data_ts": "01_data.ipynb",
         "get_event_data": "01_data.ipynb",
         "expand_filter": "02_models.ipynb",
         "ChLin": "02_models.ipynb",
         "FireHR": "02_models.ipynb",
         "download_model_weights": "02_models.ipynb",
         "load_pretrained_model": "02_models.ipynb",
         "get_preds": "03_predict.ipynb",
         "firehr_from_banet_events": "04_cli.ipynb"}

modules = ["data.py",
           "models.py",
           "predict.py",
           "cli.py"]

doc_url = "https://mnpinto.github.io/FireHR/"

git_url = "https://github.com/mnpinto/FireHR/tree/master/"

def custom_doc_links(name): return None
